/*
 * This file is part of the StorkPDF software library.
 * Copyright (c) 2010,2011 European Commission
 * Authors: Indra (www.indra.es), et al.
 *
 * This work  is Open Source and licensed by the European Commission under the
 * conditions of the Affero General Public License v3 
 * (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
 * in compliance with the License. Unless required by applicable law or agreed 
 * to in writing, software distributed under the License is distributed on an 
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */
package eu.stork.support.pkcs11;

import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.Enumeration;
import java.util.InvalidPropertiesFormatException;
import java.util.Properties;
import java.util.logging.Logger;




/**
 * The class PKCS11Support. This class adds a PKCS11 extension for using Stork PDF. This class encapsulates the required data to create signed PDF documents using PKCS11 Standard.  
 *
 */
public final class PKCS11Support {

	/** The logger. */
	private static final Logger LOG = Logger.getLogger(PKCS11Support.class
			.getName());
	/**
	 * The private key which will by used in the signature process.
	 */
	private PrivateKey privateKey;
	
	/**
	 * The certificate chain used to sign the document.. 
	 */
	private X509Certificate certificateChain[];

	/**
	 * @return the Private key.
	 */
	public PrivateKey getPrivateKey() {
		return privateKey;
	}

	/**
	 * Sets the Private key. 
	 * @param privateKey the Private key. 
	 */
	public void setPrivateKey(PrivateKey privateKey) {
		this.privateKey = privateKey;
	}

	public void setCertificateChain(X509Certificate[] certificateChain) {
		this.certificateChain = certificateChain;
	}



	private KeyStore keyStore = null;



	/**
	 * Constructor. 
	 * @param fileConf. Constructs PKCS11Support instance using a configuration file XML PKCS11 standard.
	 */
	public  PKCS11Support(String fileConf){
		init(fileConf);
		loadCryptographicServiceProvider();
		extractCertificate();
	}


	/** The HW sign prop. */
	private Properties properties = null;


	/* (non-Javadoc)
	 * @see eu.stork.peps.auth.engine.core.SAMLEngineSignI#init(java.lang.String)
	 */
	public void init(String fileConf)  {
		final InputStream is = PKCS11Support.class		
		.getResourceAsStream("/" + fileConf);
		properties = new Properties();
		try {
			properties.loadFromXML(is);
			
		} catch (InvalidPropertiesFormatException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/* (non-Javadoc)
	 * @see eu.stork.peps.auth.engine.core.SAMLEngineSignI#getCertificate()
	 */
	public X509Certificate[] getCertificateChain() {
		return certificateChain;
	}

	/* (non-Javadoc)
	 * @see eu.stork.peps.auth.engine.core.SAMLEngineSignI#sign(org.opensaml.common.SignableSAMLObject)
	 */
	private void extractCertificate()  {

		try {
			LOG.info("Star procces of sign");
			char [] pin = properties.getProperty("keyPassword").toCharArray();			

			keyStore.load(null, pin);

			String serialNumber = properties.getProperty("serialNumber");
			String issuer = properties.getProperty("issuer");

			String alias = null;
			String aliasCert = null;
			certificateChain = null;


			for (Enumeration<String> e = keyStore.aliases(); e.hasMoreElements(); ) {
				aliasCert = e.nextElement();
				certificateChain = (X509Certificate[]) keyStore.getCertificateChain(aliasCert);
				// Verified serial number, issuer
				for (int i=0;i<certificateChain.length;i++) {
					String sn = certificateChain[i].getSerialNumber().toString(16);
					String isu = certificateChain[i].getIssuerDN().getName();
					if (sn.toUpperCase().equalsIgnoreCase(serialNumber) && isu.equalsIgnoreCase(issuer)) {				
						alias = aliasCert;
						break;
					}
				}
					

				
			}

			certificateChain = (X509Certificate[]) keyStore.getCertificateChain(alias);
			privateKey = (PrivateKey) keyStore.getKey(alias, pin);

		} catch (Exception e) {
			e.printStackTrace();
		}
			

		
	}

	

	
	private void loadCryptographicServiceProvider() {
		try {		
			LOG.info("Load Cryptographic Service Provider");

			final InputStream inputStream = PKCS11Support.class		
			.getResourceAsStream("/"+properties.getProperty("configurationFile"));

			Provider pkcs11Provider =  new sun.security.pkcs11.SunPKCS11(inputStream);

			inputStream.close();

			if (Security.getProperty(pkcs11Provider.getName()) == null)
				Security.insertProviderAt(pkcs11Provider, Security.getProviders().length);

			keyStore = KeyStore.getInstance(properties
					.getProperty("keystoreType"));

		}catch(Exception e){
			e.printStackTrace();
			
		}
	}
	
//	public static void main(String[] args) {
//		PKCS11Support sign = new PKCS11Support("SignModule_P11.xml");
//		
//		sign.loadCryptographicServiceProvider();
//		
//		sign.extractCertificate();
//		
//	}

}
