﻿// This file is part of the StorkPDF software library.
// Copyright (c) 2010,2011 European Commission
// Authors: Agência para a Modernização Administrativa (www.ama.pt), et al.
//
// This work  is Open Source and licensed by the European Commission under the
// conditions of the Affero General Public License v3 
// (http://www.gnu.org/licenses/agpl.html ); you may not use this file except 
// in compliance with the License. Unless required by applicable law or agreed 
// to in writing, software distributed under the License is distributed on an 
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
// or implied. See the License for the specific language governing permissions
// and limitations under the License.

using System;
using System.Collections.Generic;
using System.Text;
using iTextSharp.text.pdf;
using System.IO;
using iTextSharpSign;
using System.Collections;
using Org.BouncyCastle.Crypto.Tls;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography;
using Org.BouncyCastle.X509;
using System.Security.Permissions;


namespace PdfGenerator
{
    public class StorkPDF : IPDFComponent
    {
        #region Properties

        private PDFSigner pdfSigner;

        private string stork_eIdentifier;

        public string Stork_eIdentifier
        {
            get { return stork_eIdentifier; }
            set { stork_eIdentifier = value; }
        }
        private string stork_surname;

        public string Stork_surname
        {
            get { return stork_surname; }
            set { stork_surname = value; }
        }
        private string stork_givenName;

        public string Stork_givenName
        {
            get { return stork_givenName; }
            set { stork_givenName = value; }
        }
        private string stork_dateOfBirth;

        public string Stork_dateOfBirth
        {
            get { return stork_dateOfBirth; }
            set { stork_dateOfBirth = value; }
        }
        private string stork_textResidence;

        public string Stork_textResidence
        {
            get { return stork_textResidence; }
            set { stork_textResidence = value; }
        }
        private string stork_countryCode;

        public string Stork_countryCode
        {
            get { return stork_countryCode; }
            set { stork_countryCode = value; }
        }
        private string stork_signature;

        public string Stork_signature
        {
            get { return stork_signature; }
            set { stork_signature = value; }
        }

        #endregion


        #region Methods

        public StorkPDF()
        { }

        /// <summary>
        /// Constructor to StorkPDF object
        /// </summary>
        /// <param name="eIdentifier_">Personal Stork eIdentifier</param>
        /// <param name="givenName_">First User Name</param>
        /// <param name="surname_">User Surname</param>
        /// <param name="textResidence_">User Address</param>
        /// <param name="dateOfBirth_">User Birthday´s</param>
        /// <param name="countryCode_">User country code´s</param>
        public StorkPDF(string eIdentifier_, string givenName_, string surname_, string textResidence_, string dateOfBirth_, string countryCode_)
        {
            this.stork_eIdentifier = eIdentifier_;
            this.stork_givenName = givenName_;
            this.stork_surname = surname_;
            this.stork_textResidence = textResidence_;
            this.stork_dateOfBirth = dateOfBirth_;
            this.stork_countryCode = countryCode_;
        }

        /// <summary>
        /// Method to write a PDF field
        /// </summary>
        /// <param name="fieldToWrite">Field name</param>
        /// <param name="fieldValue">Field value</param>
        /// <param name="pdfTemplate_">PDF template document </param>
        /// <param name="newFile_">PDF filled with all fields</param>
        /// <param name="pdfStamper_">PDF Stamper object</param>
        public void WriteField(string fieldToWrite, string fieldValue, PdfStamper pdfStamper_)
        {          
            AcroFields pdfFormFields = pdfStamper_.AcroFields;
            pdfFormFields.SetField(fieldToWrite, fieldValue);
        }

        /// <summary>
        /// Method to read from a PDF field
        /// </summary>
        /// <returns>field name to read</returns>
        public string ReadField(string fieldToRead, PdfStamper pdfStamper_)
        {
            AcroFields pdfFormFields = pdfStamper_.AcroFields;
            return pdfFormFields.GetField(fieldToRead);
        }


        /// <summary>
        /// Method to sig document
        /// </summary>
        /// <param name="certPath_">Path to cerificate file</param>
        /// <param name="pdfFilled_">PDF document with all fields filled</param>
        /// <param name="pdfToSign_">PDF to sign</param>
        /// <returns></returns>
        public bool SignPDFDocument(string certPath_, string pdfFilled_, string pdfToSign_)
        {
            Cert myCert = null;
            try
            {
                //Hard Coded.
                myCert = new Cert(certPath_, "teste");                
            }
            catch (Exception ex)
            {
                //TODO
                return false;
            }

            //Adding Meta Data
            MetaData metadata = new MetaData();
            metadata.Author = "Test Service Provider";
            metadata.Producer = "Service Provider";
            metadata.Subject = "Change of address";
            metadata.Title = "Change of address";

            try
            {
                pdfSigner = new PDFSigner(pdfFilled_, pdfToSign_, myCert,metadata);

                pdfSigner.Sign("I am the author", "PhoneNumber", "Office", true);
            }
            catch(Exception ex)
            {
                //TODO
                return false;
            }

            return true;
        }


        /// <summary>
        /// Method to check the document validity
        /// </summary>
        /// <returns>PdfReader with 'PDF Signed' Object</returns>
        public bool CheckPDFDocumentSignature(PdfReader pr_)
        {     
                PdfPKCS7 pk= null;

                if (pr_ != null)
                {
                    AcroFields af = pr_.AcroFields;
                    if (af != null)
                    {
                        ArrayList names = af.GetSignatureNames();
                        for (int k = 0; k < names.Count; ++k)
                        {
                            String name = (String)names[k];

                            //Verify if signature covers whole document
                            if (af.SignatureCoversWholeDocument(name))
                            {
                                // Verify document signature and if document was changed
                                pk = af.VerifySignature(name);
                                DateTime cal = pk.SignDate;

                                if (pk.Certificates[0].IsValid(cal))
                                    if (!pk.Verify())
                                        return false;

                                //
                                Object fails = PdfPKCS7.VerifyCertificate(pk.Certificates[0], null, cal);

                                if (fails != null)
                                    return false;

                                byte[] myCertificateBytes = pk.Certificates[0].GetTbsCertificate();
                                X509Certificate2 cert = new X509Certificate2(myCertificateBytes);

                                //Verify if PDF certificate is in my MS Thrust List of Certificates
                                X509Store store = new X509Store(StoreName.My, StoreLocation.CurrentUser);
                                store.Open(OpenFlags.OpenExistingOnly | OpenFlags.ReadOnly);

                                if (store.Certificates.Contains(cert))
                                    return true;
                                else
                                    return false;
                            }
                        }
                    }
                }

            return false;
        }

        #endregion
    }
}
